﻿

namespace $safeprojectname$
{
    using System;
    using System.ComponentModel.Composition;
    using System.IO;
    using System.Linq;
    using System.Windows.Forms;
    using EnvDTE;
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.TeamArchitect.PowerTools;
    using Microsoft.VisualStudio.TeamArchitect.PowerTools.Features;
    using Microsoft.VisualStudio.TeamArchitect;
    using System.ComponentModel;
    using System.Windows.Input;

    /// <summary>
    /// Command used by Gudiance Adorner to execute commands in the corresponding Feature Extension
    /// </summary>
    [DisplayName("AdornmentGatewayCommand")]
    [Description("This command is used by the GuidanceTag Adorner to execute commands")]
    public class AdornmentGatewayCommand : FeatureCommand
    {
        // Declared public properties will be exposed in the Command Bindings property

        EnvDTE.DTE dte;

        /// <summary>
        /// Gets the Feature Extension instance for this invocation
        /// of the Condition
        /// </summary>
        [Import(AllowDefault = true)]
        public IFeatureExtension Feature { get; private set; }

        /// <summary>
        /// Gets and sets the UriReferenceService using MEF
        /// The UriReferenceService takes, amongst other things, content:// URIs
        /// and can retrieve either the path or contents of the specified file as desired
        /// </summary>
        [Import]
        public IFxrUriReferenceService UriReferenceService { get; set; }

        /// <summary>
        /// Gets and sets the ServiceProvider using MEF
        /// </summary>
        [Import]
        public SVsServiceProvider ServiceProvider { get; set; }

        /// <summary>
        /// Gets and sets the ISolution using MEF
        /// </summary>
        [Import]
        public ISolution Solution { get; set; }

        [Import]
        public IFeatureManager theFeatureManager { get; set; }


        /// <summary>
        /// Called when any launch point bound to a command binding which refers to this command
        /// is invoked
        /// </summary>
        /// <param name="feature">Instance of Feature Extension which is invoking the command</param>
        public override void Execute()
        {
            dte = ServiceProvider.GetService(typeof(DTE)) as EnvDTE.DTE;

            ExecuteCommand ec = new ExecuteCommand(theFeatureManager);
            

            string adormentCommand = BlackboardManager.Current.Get("AdornmentCommand");
            switch (adormentCommand)
            {
                case "CursorDownThenPaste":
                    string pasteDoneCheck = BlackboardManager.Current.Get("PasteCompleted");
                    if (pasteDoneCheck != null && pasteDoneCheck != "Paste.Done")
                    {
                        ec.VSCommand("Edit.LineOpenBelow", null, dte);
                        ec.VSCommand("Edit.Paste", null, dte);
                        BlackboardManager.Current.Set("PasteCompleted", "Paste.Done");
                        int oldCurLine = int.Parse(BlackboardManager.Current.Get("CurrentLine"));

                        ((TextSelection) dte.ActiveDocument.Selection).MoveTo(oldCurLine,1);
                    }
                    break;

                case "CloseAndNavigate":
                    bool pasteHasBeenPerformed = BlackboardManager.Current.Get("PasteCompleted") != null &&
                                                 BlackboardManager.Current.Get("PasteCompleted") == "Paste.Done";

                    //
                    // First, let's "nuke" the adornment tag so we don't see
                    // the UX next time we enter the code (and the search will work properly!
                    //
                    //ec.VSCommand("Edit.FormatDocument", null, dte);
                    System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

                    Find findObject;

                    findObject = dte.Find;

                    findObject.FindReplace(vsFindAction.vsFindActionReplaceAll,
                        BlackboardManager.Current.Get("LastSubstitutionTag"),
                        0,

                        //
                        // If the paste has been done, we nuke the initial string, otherwise we put it back
                        //
                        pasteHasBeenPerformed ? "" : BlackboardManager.Current.Get("InitialReplacementString"),
                        vsFindTarget.vsFindTargetCurrentDocument,
                        "",
                        "",
                        vsFindResultsLocation.vsFindResultsNone);

                    findObject.Action = vsFindAction.vsFindActionFind;

                    findObject.Backwards = false;
                    findObject.FindWhat = BlackboardManager.Current.Get("LastSubstitutionTag");
                    findObject.KeepModifiedDocumentsOpen = true;
                    findObject.MatchCase = true;
                    findObject.PatternSyntax = vsFindPatternSyntax.vsFindPatternSyntaxLiteral;
                    findObject.ResultsLocation = vsFindResultsLocation.vsFindResultsNone;
                    findObject.Target = vsFindTarget.vsFindTargetCurrentDocument;
                    findObject.Execute();

                    ec.VSCommand("File.SaveAll", null, dte);
                    ec.VSCommand("Window.CloseDocumentWindow", null, dte);


                    string currentFeature = Feature.FeatureId;

                    //
                    // Now, let's look on the Blackboard to see if there's a new guidance topic
                    // to which we need to navigate
                    //

                    string newTopic = BlackboardManager.Current.Get("NextGuidanceTopic");

                    if (newTopic != null && newTopic.Length > 0)
                    {
                        BlackboardManager.Current.Set((Feature.GuidanceWorkflow.FocusedAction.Name + "Completed").Replace(" ", ""), "True");
                        FeatureCallContext.Current.GuidanceBrowserControl.HREF = new Uri("content://" + currentFeature + "/Content/ProcessGuidance/" + newTopic);
                    }

                    BlackboardManager.Current.Set("NextGuidanceTopic", "");
                    BlackboardManager.Current.Set("LastSubstitutionTag", "");

                    System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;

                    break;
            }
        }
    }
}
