﻿using System;
using System.ComponentModel.Composition;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using EnvDTE;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.TeamArchitect.PowerTools;
using Microsoft.VisualStudio.TeamArchitect.PowerTools.Features;
using Microsoft.VisualStudio.TeamArchitect;
using System.ComponentModel;
using System.Windows.Input;


// <copyright file="ExecuteCommand.cs">
// Copyright (c) 2011 All Rights Reserved
// </copyright>
// <author>Michael Lehman (DreamTimeStudioZ)</author>
// <date>2/8/2011 4:41:41 PM</date>
// <summary>Feature Extension Command that does X</summary>
namespace $safeprojectname$
{


	/// To use, add a hyperlink in one of your guidance documents like this:
	///     launchpoint://<yourFXname>/ExecuteCommand#<cmd>|<optional-args>
	///
	/// Example:
	///     launchpoint://InstantFeatureExtension1/ExecuteCommand#OpenFile|*\MyClass.cs
	///
    [LaunchPoint(Id = "ExecuteCommand")]
    public partial class ExecuteCommand : LinkLaunchPoint
    {
        // Declared public properties will be exposed in the Command Bindings property

        EnvDTE.DTE dte;

        [ImportingConstructor]
        public ExecuteCommand(IFeatureManager featureManager)
            : base(featureManager)
        {
        }

        /// <summary>
        /// Gets and sets the UriReferenceService using MEF
        /// The UriReferenceService takes, amongst other things, content:// URIs
        /// and can retrieve either the path or contents of the specified file as desired
        /// </summary>
        [Import]
        public IFxrUriReferenceService UriReferenceService { get; set; }

        /// <summary>
        /// Gets and sets the ServiceProvider using MEF
        /// </summary>
        [Import]
        public SVsServiceProvider ServiceProvider { get; set; }

        /// <summary>
        /// Gets and sets the ISolution using MEF
        /// </summary>
        [Import]
        public ISolution Solution;

        /// <summary>
        /// Called when any launch point bound to a command binding which refers to this command
        public override bool CanExecute(IFeatureExtension feature)
        {
            return true;
        }
        /// is invoked
        /// </summary>
        /// <param name="feature">Instance of Feature Extension which is invoking the command</param>
        public override void Execute(IFeatureExtension feature)
        {
            //
            // We use a little Feature Extention Runtime magic here to retrieve the
            // current URI being processed by the Guidance Workflow web browser
            string[] parts;
            parts = FeatureCallContext.Current.GuidanceBrowserControl.CurrentLink.ToString().Split('#');

            string[] args;

            //
            // If there is no "fragment part" then we do nothing
            if (parts != null && parts.Length > 0)
            {
                args = parts[1].Split('|');

                //
                // Uncomment the line below to debug any new commands you add
                //
                //System.Windows.Forms.MessageBox.Show("There are " + args.Length + " args which are: " + parts[1]);

                dte = ServiceProvider.GetService(typeof(DTE)) as EnvDTE.DTE;
                if (dte == null)
                    return;

                //
                // Ok, here's where the URI hits the "road"
                // switch based on the "command" part
                //

                switch (args[0].ToLower())
                {
                    case "openfile":
                        if (args.Length > 2)
                            OpenFile(args[1].Replace("/", "\\").Replace("%20", " ").Replace("%25", "%"),
                                     args[2],
                                     null,
                                     dte,
                                     Solution);
                        else
                            OpenFile(args[1].Replace("/", "\\").Replace("%20", " ").Replace("%25", "%"),
                                null,
                                null,
                                dte,
                                Solution);
                        break;

                    case "buildandrun":
                        VSCommand("Debug.Start", null, dte);
                        break;

                    case "nextstep":
                        BlackboardManager.Current.Set((feature.GuidanceWorkflow.FocusedAction.Name + "Completed").Replace(" ", ""), "True");
                        FeatureCallContext.Current.GuidanceBrowserControl.HREF = new Uri("content://" + feature.FeatureId + "/Content/ProcessGuidance/" + args[1]);
                        break;


                    //
                    // The swiss army knife of commands (for great HoL and tutorials)
                    // This command can have between 3 and 7 parameters as follows:
                    //
                    // Arg 1 - Name of the file to copy (must be in the Content\Data directory)
                    // Arg 2 - Name of the file to open (should be a path within the currently open solution: e.g. */Class1.cs)
                    // Arg 3 - A string to "find" i.e. - position the open document wherever the first instance of this arg is found
                    // Arg 4 - (OPTIONAL) Name of the "next" guidance topic to jump to
                    // Arg 5 - (OPTIONAL) Data to put on Blackboard under key "OptionalArgs" - used by editor Adornments (often used for MP3 file)
                    // Arg 6 - (OPTIONAL) string to substitute for arg 3 (useful when an Editor adornment uses a generic regex mechanism)
                    // Arg 7 - (OPTIONAL) string "true" or "false" which indicates if the blackboard should be updated to indicate step is complete
                    //
                    case "copyopenfind":
                        string substituteTag = "//$safeprojectname$AdornerTAG";	// must match string in RegEx contained in GuidanceTagger.cs

                        if (args.Length < 4)
                        {
                            MessageBox.Show("CopyOpenFind requires 3,4,5 or 6 parameters");
                            break;
                        }
                        BlackboardManager.Current.Set("InitialReplacementString",args[3]);

                        CopyToClipboard(args[1], UriReferenceService, dte);

                        if (args.Length > 4 && args[4].Length > 0)
                            BlackboardManager.Current.Set("NextGuidanceTopic", args[4]);
                        else
                            BlackboardManager.Current.Set("NextGuidanceTopic", "");

                        if (args.Length > 5 && args[5].Length > 0)
                        {
                            var fullPath = UriReferenceService.ResolveUri<FeatureContent>(
                                new Uri(string.Format("content://./Content/Data/{0}", args[5]))).Path;
                            BlackboardManager.Current.Set("OptionalArgs", fullPath);
                        }
                        else
                            BlackboardManager.Current.Set("OptionalArgs", "");



                        if (args.Length > 6 && args[6].Length > 0)
                            substituteTag = args[6];


                        if (args.Length > 7 && args[7].Length > 0)
                            BlackboardManager.Current.Set("CompleteStep", "True");
                        else
                            BlackboardManager.Current.Set("CompleteStep", "False");


                        //
                        // We set this so the editor Adorner can zap it on exit
                        //
                        BlackboardManager.Current.Set("LastSubstitutionTag", substituteTag);

                        //y
                        // We set this to allow paste, adorner set it to "Paste.Done" to prevent double pasting
                        //
                        BlackboardManager.Current.Set("PasteCompleted", "Paste.Not.Done");

                        //
                        // Now, we're setup, go open the file (tweaking the slashes, decoding the spaces and percents in the URI)
                        //
                        OpenFile(args[2].Replace("/", "\\").Replace("%20", " ").Replace("%25", "%"),
                                 args[3],
                                 substituteTag,
                                 dte,
                                 Solution);
                        break;

                    case "vscommand":
                        if (args.Length > 2)
                            VSCommand(args[1], args[2], dte);
                        else
                            VSCommand(args[1], null, dte);
                        break;

                    case "startprocess":
                        if (args.Length > 2)
                            StartProcess(args[1], args[2], UriReferenceService, dte);
                        else
                            StartProcess(args[1], "", UriReferenceService, dte);
                        break;

                    case "copytoclipboard":
                        CopyToClipboard(args[1], UriReferenceService, dte);
                        break;

                    case "showmessage":
                        ShowMessage(args[1]);
                        break;

                    case "showmethod":
                        ShowMethod(args[1], args[2], dte, Solution);
                        break;

                    default:
                        MessageBox.Show("Unknown command selected: " + args[0]);
                        break;
                }
            }

        }

        /// <summary>
        /// Show Method - using code model, find and position to a method by name
        /// </summary>
        /// <param name="filePath">Path of file to search (e.g. myFile.cpp)</param>
        /// <param name="methodName">Name of method to search for</param>
        /// <param name="dte">Reference to EnvDTE object</param>
        /// <param name="Solution">Reference to ISolution (MEF import)</param>
        public void ShowMethod(string filePath, string methodName, EnvDTE.DTE dte, ISolution Solution)
        {
            string realFilename = filePath.Replace("/", "\\").Replace("%20", " ").Replace("%25", "%");
            var fileItem = this.Solution.Find(realFilename).FirstOrDefault();
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

            if (fileItem != null)
            {
                FindAndShowMethod(fileItem, methodName, dte);
            }
            else
                MessageBox.Show("Unable to find a file that matches: " + filePath);
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;

        }

        /// <summary>
        /// Implement a recursive search using the code model for a method
        /// </summary>
        /// <param name="incomingItem">Container in which to search</param>
        /// <param name="methodName">Name of method to find</param>
        /// <param name="dte">Reference to EnvDTE</param>
        void FindAndShowMethod(IItemContainer incomingItem, string methodName, EnvDTE.DTE dte)
        {

            if (incomingItem == null)
                return;

            ProjectItem pi = incomingItem.As<ProjectItem>();

            if (pi != null)
            {
                FileCodeModel itemModel = pi.FileCodeModel;
                if (itemModel != null)
                {
                    foreach (CodeElement ce in itemModel.CodeElements)
                    {
                        if ((ce.Kind == vsCMElement.vsCMElementFunction) &&
                            (ce.Name == methodName))
                        {
                            CodeFunction cf = ce as CodeFunction;
                            string physPath = incomingItem.PhysicalPath;
                            dte.StatusBar.Text = "Opening file " + incomingItem.Name;
                            dte.ItemOperations.OpenFile(physPath);
                            dte.StatusBar.Text = "Ready";
                            TextSelection s = dte.ActiveDocument.Selection as TextSelection;
                            s.MoveToPoint(cf.StartPoint);
                        }


                    }
                }

            }
        }


        /// <summary>
        /// Finds a file, based on the supplied filepath and then
        /// asks VS to open it in the default editor associated with that type of file
        /// </summary>
        /// <param name="filepath">String containing file path (may include wildcards)</param>
        public void ShowMessage(string messageToShow)
        {
            MessageBox.Show(messageToShow);
        }

        /// <summary>
        /// Finds a file, based on the supplied filepath and then
        /// asks VS to open it in the default editor associated with that type of file
        /// If the searchKey is non-null, then a Find command is also executed
        /// </summary>
        /// <param name="filepath">String containing file path (may include wildcards)</param>
        /// <param name="searchKey">String containing a string to search for (or null)</param>
        public void OpenFile(string filepath, string searchKey, string replaceByKey, EnvDTE.DTE dte, ISolution Solution)
        {
            var fileItem = this.Solution.Find(filepath).FirstOrDefault();
            Window openedFileWindow;
            Find findObject;

            if (fileItem != null)
            {
                dte.StatusBar.Text = "Opening file " + fileItem.Name;
                System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

                openedFileWindow = dte.ItemOperations.OpenFile(fileItem.PhysicalPath);

                if (openedFileWindow != null && searchKey != null)
                {
                    findObject = openedFileWindow.Document.DTE.Find;

                    if (replaceByKey != null)
                    {
                        findObject.FindReplace(vsFindAction.vsFindActionReplaceAll,
                            searchKey.Replace("~", "#"),
                            0,

                            replaceByKey,
                            vsFindTarget.vsFindTargetCurrentDocument,
                            "",
                            "",
                            vsFindResultsLocation.vsFindResultsNone);
                        searchKey = replaceByKey;
                    }

                    findObject.Action = vsFindAction.vsFindActionFind;

                    findObject.Backwards = false;
                    findObject.FindWhat = searchKey.Replace("~", "#");
                    findObject.KeepModifiedDocumentsOpen = true;
                    findObject.MatchCase = true;
                    findObject.PatternSyntax = vsFindPatternSyntax.vsFindPatternSyntaxLiteral;
                    findObject.ResultsLocation = vsFindResultsLocation.vsFindResultsNone;
                    findObject.Target = vsFindTarget.vsFindTargetCurrentDocument;
                    findObject.Execute();

                    TextSelection selection = (TextSelection)openedFileWindow.Document.Selection;
                    int currentLine = selection.CurrentLine + 1;
                    BlackboardManager.Current.Set("CurrentLine", currentLine.ToString());
                }

                System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;
                dte.StatusBar.Text = "Ready";
            }
            else
                MessageBox.Show("Unable to find a file that matches: " + filepath);
        }

        /// <summary>
        /// Execute arbitrary Visual Studio Command with optional arguments.
        /// A list of commands can be found in the Tools --> Options --> Keyboard dialog or
        /// on MSDN
        /// </summary>
        /// <param name="CommandName">Command string for Visual Studio</param>
        /// <param name="CommandArgs">Optional arguments</param>
        public void VSCommand(string CommandName, string CommandArgs, EnvDTE.DTE dte)
        {
            dte.StatusBar.Text = string.Format("Executing command {0} {1}", CommandName, CommandArgs);
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

            int deadmanCounter = 100000;

            if (CommandArgs != null)
            {
                while (deadmanCounter-- > 0)
                {
                    try
                    {
                        dte.ExecuteCommand(CommandName, CommandArgs);
                        break;
                    }
                    catch
                    {
                    }
                }

                if (deadmanCounter <= 0)
                    MessageBox.Show("Unable to execute: " + CommandName + "(" + CommandArgs + ")");
            }
            else
            {
                while (deadmanCounter-- > 0)
                {
                    try
                    {
                        dte.ExecuteCommand(CommandName);
                        break;
                    }
                    catch
                    {
                    }
                }

                if (deadmanCounter <= 0)
                    MessageBox.Show("Unable to execute: " + CommandName);
            }
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;
            dte.StatusBar.Text = "Ready";

        }

        /// <summary>
        /// Starts a process with optional arguments.
        /// For example, to play a video, simply supply the path to the video as the file name
        /// and Windows Media Player will be started.
        /// </summary>
        /// <param name="FileName">Name of process to start or name of file to auto-start</param>
        /// <param name="Arguments">Optional list of arguments</param>
        public void StartProcess(string FileName, string Arguments, IFxrUriReferenceService UriReferenceService, EnvDTE.DTE dte)
        {
            string filePath = null;
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

            try
            {
                filePath = UriReferenceService.ResolveUri<FeatureContent>(
                new Uri(string.Format("content://./Content/Data/{0}", FileName))).Path;
            }
            catch
            {
            }

            if (filePath != null)
            {
                dte.StatusBar.Text = string.Format("Starting process {0} {1}", FileName, Arguments);
                System.Diagnostics.Process.Start(filePath, Arguments);
            }
            else
            {
                string realFilename = FileName.Replace("/", "\\").Replace("%25", "%");

                dte.StatusBar.Text = string.Format("Starting process {0} {1}", realFilename, Arguments);
                System.Diagnostics.Process.Start(FileName, Arguments);
            }
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.Default;
            dte.StatusBar.Text = "Ready";
        }

        /// <summary>
        /// Copy one of the files from the Content\Data directory to the VS clipboard
        /// and tell the user it's been done in the status bar
        /// </summary>
        /// <param name="fileToCopy">Name of the file to copy from the Content\Data directory</param>
        public void CopyToClipboard(string fileToCopy, IFxrUriReferenceService UriReferenceService, EnvDTE.DTE dte)
        {
            System.Windows.Forms.Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

            var txtPath = UriReferenceService.ResolveUri<FeatureContent>(
                new Uri(string.Format("content://./Content/Data/{0}", fileToCopy))).Path;
            var content = File.ReadAllText(txtPath);
            Clipboard.SetText(content);

            dte.StatusBar.Text = "Data copied to clipboard";
        }

        /// <summary>
        /// Gets the name of the binding associated with this launch point
        /// </summary>
        protected override string BindingName
        {
            //
            // Since we override the Execute() method, we don't need to return
            // anything meaningful here, so we return the empty string
            //
            get 
            { 
                return String.Empty; 
            }
        }
    }
}