using System;
using System.ComponentModel.Composition;
using Microsoft.VisualStudio.TeamArchitect.PowerTools.Features;
using Microsoft.VisualStudio.TeamArchitect.PowerTools;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using System.Linq;
using System.Collections.Generic;
using System.IO;
using System.Diagnostics;

namespace $safeprojectname$
{
    /// <summary>
    /// Used to return top and bottom nodes of a sub-folder
    /// </summary>
    class SubDirData
    {
        public Fork fork;
        public Join join;
    }


    public partial class ProcessWorkflow : GuidanceWorkflow
    {
        [Import]
        private IFeatureCompositionService FeatureComposition { get; set; }

        [Import]
        private IFxrUriReferenceService UriReferenceService { get; set; }

        /// <summary>
        /// Force all the workflow nodes to be "green" (i.e. Ready)
        /// </summary>
        public override bool IgnorePostConditions
        {
            get
            {
                return true;
            }
        }

        partial void OnPostInitialize()
        {
            GuidanceWorkflow tempWF = new GuidanceWorkflow();

            Node endOfList = null;
            //
            // Ok, we essentially ignore the auto-generated code and build our own the way we want :)
            //
            tempWF.Name = "GuidanceWorkflow";

            var initial = new Initial
            {
                Name = "Initial1",
            };

            string nodeName = string.Empty;
            string nodeLink = string.Empty;

            //
            // Note:  We couldn't use the content://./Content/... form of the URI
            // because our feature it not activated at the time this code runs
            //
            string pathToDir = this.UriReferenceService
    .ResolveUri<FeatureContent>(new Uri("content://$safeprojectname$/Content/Runtime/FeatureExtensionRuntimePro.vsix")).Path;
            pathToDir = Path.Combine(Directory.GetParent(Path.GetDirectoryName(pathToDir)).FullName, "ProcessGuidance");

            List<string> sortedNames = Directory.GetFileSystemEntries(pathToDir).ToList();
            sortedNames.Sort();

            foreach (string theFn in sortedNames)
            {
                string fn = theFn;

                if (Directory.Exists(fn))
                {
                    List<string> sortedFiles = Directory.GetFileSystemEntries(fn).ToList();
                    if (sortedFiles.Count == 0)
                        continue;

                    sortedFiles.Sort();

                    //
                    // Create a fork/join and encapsulate the results in the FJ
                    //
                    string forkName = Path.GetFileNameWithoutExtension(fn);

                    if (forkName.LastIndexOf('~') > 0)
                    {
                        forkName = forkName.Substring(forkName.LastIndexOf('~') + 1);
                    }

                    //Debug.WriteLine("Creating Fork:" + forkName);
                    Fork fork = new Fork
                    {
                        Name = forkName,
                    };

                    //Debug.WriteLine("Creating Join:" + Path.GetFileNameWithoutExtension(fn) + "_join");
                    Join join = new Join
                    {
                        Name = Path.GetFileNameWithoutExtension(fn) + "_join",
                    };

                    IGuidanceAction lastGuidanceAction = fork;

                    foreach (string theDocName in sortedFiles)
                    {

                        if (Directory.Exists(theDocName))
                        {
                            SubDirData subDirData;
                            subDirData = ProcessSubDir(theDocName, pathToDir);
                            fork.ConnectTo(subDirData.fork);
                            subDirData.join.ConnectTo(join);
                        }
                        else
                        {
                            string docName = theDocName;

                            if (docName.LastIndexOf('~') > 0)
                            {
                                docName = docName.Substring(docName.LastIndexOf('~') + 1);
                            }

                            if (docName.ToLower().EndsWith(".link") ||
                                docName.ToLower().EndsWith(".url"))
                            {
                                if (Path.GetFileNameWithoutExtension(theDocName).StartsWith("~") &&
                                    Path.GetFileNameWithoutExtension(theDocName).Substring(1).ToLower() == forkName.ToLower())
                                {
                                    fork.Link = File.ReadAllText(theDocName).Replace("\r\n", "");
                                }
                                else
                                {
                                    //Debug.WriteLine("Creating node:" + Path.GetFileNameWithoutExtension(docName) );
                                    var newGuidanceAction = new GuidanceAction
                                    {
                                        Name = Path.GetFileNameWithoutExtension(docName),
                                        Link = GetLinkFromFile(theDocName),
                                    };

                                    //Debug.WriteLine("Connecting: " + lastGuidanceAction.Name + " to " + newGuidanceAction.Name);
                                    if (lastGuidanceAction == fork)
                                        fork.ConnectTo(newGuidanceAction);
                                    else
                                        ((GuidanceAction)lastGuidanceAction).ConnectTo(newGuidanceAction);

                                    lastGuidanceAction = newGuidanceAction;
                                }
                            }
                            else if (docName.ToLower().EndsWith(".mht") ||
                                     docName.ToLower().EndsWith(".txt") ||
                                     docName.ToLower().EndsWith(".html"))
                            {
                                if (Path.GetFileNameWithoutExtension(theDocName).StartsWith("~") &&
                                    Path.GetFileNameWithoutExtension(theDocName).Substring(1).ToLower() == forkName.ToLower())
                                {
                                    fork.Link = "content://$safeprojectname$/Content/ProcessGuidance/" +
                                        fn.Replace(pathToDir, "").Substring(1).Replace("\\", "/") +
                                    "/" +
                                    Path.GetFileName(theDocName);
                                }
                                else
                                {
                                    var newGuidanceAction = new GuidanceAction
                                    {
                                        Name = Path.GetFileNameWithoutExtension(docName),
                                        Link = "content://$safeprojectname$/Content/ProcessGuidance/" +
                                        fn.Replace(pathToDir, "").Substring(1).Replace("\\", "/") +
                                        "/" +
                                        Path.GetFileName(theDocName),
                                    };

                                    //Debug.WriteLine("Connecting: " + lastGuidanceAction.Name + " to " + newGuidanceAction.Name);
                                    if (lastGuidanceAction == fork)
                                        fork.ConnectTo(newGuidanceAction);
                                    else
                                        ((GuidanceAction)lastGuidanceAction).ConnectTo(newGuidanceAction);

                                    lastGuidanceAction = newGuidanceAction;
                                }
                            }
                        }
                    }

                    if (lastGuidanceAction == fork)
                        System.Windows.Forms.MessageBox.Show("Invalid Guidance Configuration.  Directory " + fork.Name +
                                                             " is empty.");
                    else
                        ((GuidanceAction)lastGuidanceAction).ConnectTo(join);


                    if (initial.Successors.ToList().Count == 0)
                    {
                        initial.ConnectTo(fork);
                    }
                    else
                    {
                        if (endOfList is Join)
                            ((Join)endOfList).ConnectTo(fork);
                        else
                            ((GuidanceAction)endOfList).ConnectTo(fork);
                    }
                    endOfList = join;
                }
                else
                {
                    //
                    // Not a directory, process an individual file
                    //
                    GuidanceAction newGuidanceAction = null;
                    string docName = Path.GetFileNameWithoutExtension(fn);

                    if (docName.LastIndexOf('~') > 0)
                    {
                        docName = docName.Substring(docName.LastIndexOf('~') + 1);
                    }
                    //
                    // It's just a file
                    //
                    if (fn.ToLower().EndsWith(".url") ||
                        fn.ToLower().EndsWith(".link"))
                    {
                        newGuidanceAction = new GuidanceAction
                        {
                            Name = Path.GetFileNameWithoutExtension(docName),
                            Link = GetLinkFromFile(fn),
                        };
                    }
                    else if (fn.ToLower().EndsWith(".mht") ||
                             fn.ToLower().EndsWith(".txt") ||
                             fn.ToLower().EndsWith(".html"))
                    {
                        newGuidanceAction = new GuidanceAction
                        {
                            Name = Path.GetFileNameWithoutExtension(docName),
                            Link = "content://$safeprojectname$/Content/ProcessGuidance/" +
                            Path.GetFileName(fn),

                        };
                    }
                    if (initial.Successors.ToList().Count == 0)
                    {
                        initial.ConnectTo(newGuidanceAction);
                        //Debug.WriteLine("Connecting: " + initial.Name + " to " + newGuidanceAction.Name);
                    }
                    else
                    {
                        if (endOfList is Join)
                        {
                            ((Join)endOfList).ConnectTo(newGuidanceAction);
                            //Debug.WriteLine("Connecting (to join): " + endOfList.Name + " to " + newGuidanceAction.Name);
                        }
                        else
                        {
                            //Debug.WriteLine("Connecting: " + endOfList.Name + " to " + newGuidanceAction.Name);
                            ((GuidanceAction)endOfList).ConnectTo(newGuidanceAction);
                        }
                    }
                    endOfList = newGuidanceAction;
                }
            }


            var final0 = new Final
            {
                Name = "ActivityFinal1",
            };


            tempWF.ConnectTo(initial);

            if (initial.Successors.ToList().Count == 0)
            {
                var guidanceAction = new GuidanceAction
                {
                    Name = "No Guidance Documents Found",
                    Link = "content://$safeprojectname$/Content/ProcessGuidance/1/Overview.mht",
                };
                initial.ConnectTo(guidanceAction);
                guidanceAction.ConnectTo(final0);
            }
            else
            {
                if (endOfList is Join)
                    ((Join)endOfList).ConnectTo(final0);
                else
                    ((GuidanceAction)endOfList).ConnectTo(final0);
            }
            INode topNode = this.Successors.ToArray<INode>()[0];
            Node originalInitialNode = topNode.Successors.ToArray<INode>()[0] as Node;

            try
            {
                if (originalInitialNode != null)
                    originalInitialNode.InsertWorkflow(tempWF, true);
                //System.Windows.Forms.MessageBox.Show("Number of nodes in workflow: " + nodeCount.ToString());
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Instant FX Runtime Error - Probable cause: an empty folder in Content\\ProcessGuidance");
            }
        }

        /// <summary>
        /// Process (recursively) sub-folders returning a SubDirData containing the fork/join nodes
        /// </summary>
        private SubDirData ProcessSubDir(string fn, string topLevelPath)
        {
            SubDirData results = new SubDirData();
            results.fork = null;
            results.join = null;

            //
            // Create a fork/join and encapsulate the results in the FJ
            //
            string forkName = Path.GetFileNameWithoutExtension(fn);

            if (forkName.LastIndexOf('~') > 0)
            {
                forkName = forkName.Substring(forkName.LastIndexOf('~') + 1);
            }

            Fork fork = new Fork
            {
                Name = forkName,
            };

            results.fork = fork;

            Join join = new Join
            {
                Name = Path.GetFileNameWithoutExtension(fn) + "_join",
            };

            results.join = join;

            List<string> sortedFiles = Directory.GetFileSystemEntries(fn).ToList();
            sortedFiles.Sort();

            IGuidanceAction lastGuidanceAction = fork;

            foreach (string theDocName in sortedFiles)
            {

                if (Directory.Exists(theDocName))
                {
                    SubDirData subDirData;
                    subDirData = ProcessSubDir(theDocName, topLevelPath);
                    fork.ConnectTo(subDirData.fork);
                    subDirData.join.ConnectTo(join);
                }
                else
                {
                    string docName = theDocName;

                    if (docName.LastIndexOf('~') > 0)
                    {
                        docName = docName.Substring(docName.LastIndexOf('~') + 1);
                    }

                    if (docName.ToLower().EndsWith(".link") ||
                        docName.ToLower().EndsWith(".url"))
                    {
                        if (Path.GetFileNameWithoutExtension(theDocName).StartsWith("~") &&
                            Path.GetFileNameWithoutExtension(theDocName).Substring(1).ToLower() == forkName.ToLower())
                        {
                            fork.Link = File.ReadAllText(docName).Replace("\r\n", "");
                        }
                        else
                        {
                            var newGuidanceAction = new GuidanceAction
                            {
                                Name = Path.GetFileNameWithoutExtension(docName),
                                Link = GetLinkFromFile(theDocName),
                            };

                            if (lastGuidanceAction == fork)
                                fork.ConnectTo(newGuidanceAction);
                            else
                                ((GuidanceAction)lastGuidanceAction).ConnectTo(newGuidanceAction);

                            lastGuidanceAction = newGuidanceAction;
                        }
                    }
                    else if (docName.ToLower().EndsWith(".mht") ||
                             docName.ToLower().EndsWith(".txt") ||
                             docName.ToLower().EndsWith(".html"))
                    {
                        if (Path.GetFileNameWithoutExtension(theDocName).StartsWith("~") &&
                            Path.GetFileNameWithoutExtension(theDocName).Substring(1).ToLower() == forkName.ToLower())
                        {
                            fork.Link = "content://$safeprojectname$/Content/ProcessGuidance/" +
                                fn.Replace(topLevelPath, "").Substring(1).Replace("\\", "/") +
                                "/" +
                            Path.GetFileName(theDocName);
                        }
                        else
                        {
                            var newGuidanceAction = new GuidanceAction
                            {
                                Name = Path.GetFileNameWithoutExtension(docName),
                                Link = "content://$safeprojectname$/Content/ProcessGuidance/" +
                                fn.Replace(topLevelPath, "").Substring(1).Replace("\\", "/") +
                                "/" +
                                Path.GetFileName(theDocName),
                            };

                            if (lastGuidanceAction == fork)
                                fork.ConnectTo(newGuidanceAction);
                            else
                                ((GuidanceAction)lastGuidanceAction).ConnectTo(newGuidanceAction);

                            lastGuidanceAction = newGuidanceAction;
                        }
                    }
                }
            }

            if (lastGuidanceAction == fork)
                System.Windows.Forms.MessageBox.Show("Invalid Guidance Configuration.  Directory " + fork.Name +
                                                     " is empty.");
            else
                ((GuidanceAction)lastGuidanceAction).ConnectTo(join);


            return results;
        }


        /// <summary>
        /// Favorite files from Internet Explorer contain not only the URL for a web page
        /// But also other information.  This method allows .link files to contain either
        /// just the URL or be drag/dropped from the IE Favorite directory
        /// </summary>
        private string GetLinkFromFile(string fn)
        {
            string fileContents = File.ReadAllText(fn);
            int start;

            if ((start = fileContents.IndexOf("[InternetShortcut]")) >= 0)
            {                                //123456789012345678
                start += 20; // Length of [InternetShortcut] + 2 for CRLF
                int end = fileContents.Substring(start).IndexOf("\r") - 1;
                return (fileContents.Substring(start, end - start));
            }
            else
                return (fileContents.Replace("\r\n", ""));
        }

    }
}

