﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Microsoft.VisualStudio.TeamArchitect.PowerTools;
using Microsoft.VisualStudio.TeamArchitect.PowerTools.Features;
using System.ComponentModel.Composition;
using Microsoft.VisualStudio.ComponentModelHost;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.ExtensibilityHosting;


namespace $safeprojectname$.UI
{
    /// <summary>
    /// Interaction logic for GuidancePanel.xaml
    /// </summary>
    public partial class GuidancePanel : UserControl, INotifyPropertyChanged
    {
        private MediaPlayer mediaPlayer;
        private MediaTimeline timeline;
        private GuidanceStepDetails stepDetails;

        private bool playing;
        public bool Playing
        {
            get { return playing; }
            private set { playing = value; NotifyPropertyChanged( "Playing" ); }
        }

        private bool pasted;
        public bool Pasted
        {
            get { return pasted; }
            private set { pasted = value; NotifyPropertyChanged( "Pasted" ); }
        }

        public GuidancePanel()
        {
            InitializeComponent();
        }

        public GuidancePanel( GuidanceTag tag )
        {
            InitializeComponent();
            
            stepDetails = tag.Details;

            mediaPlayer = new MediaPlayer();
            timeline = new MediaTimeline( stepDetails.AudioCommentary  );
            mediaPlayer.Clock = timeline.CreateClock( true ) as MediaClock;
            mediaPlayer.Clock.Controller.Stop();
            mediaPlayer.Clock.Completed += new EventHandler( Clock_Completed );
            //this.PasteButton.Visibility = System.Windows.Visibility.Visible;
            string stateOfPaste = BlackboardManager.Current.Get("PasteCompleted");

            Pasted = stateOfPaste == "Paste.Done";
        }

        void Clock_Completed( object sender, EventArgs e )
        {
            mediaPlayer.Clock.Controller.Stop();
            Playing = false;
        }

        private void PasteClick( object sender, RoutedEventArgs e )
        {
            Pasted = true;
            IFeatureExtension ourFeature = stepDetails.TheFeatureManager.ActiveFeature;

            var cb = ourFeature.Commands.FindByName("AdornmentGatewayCB");

            BlackboardManager.Current.Set("AdornmentCommand", "CursorDownThenPaste");

            cb.Evaluate();
            cb.Value.Execute();

            //this.PasteButton.IsEnabled = false;
            //this.NextButton.IsEnabled = true;
            //this.PasteButton.Visibility = System.Windows.Visibility.Hidden;
        }

        private void AudioClick( object sender, RoutedEventArgs e )
        {
            if (mediaPlayer.Clock.CurrentState == System.Windows.Media.Animation.ClockState.Active)
            {
                if ( mediaPlayer.Clock.CurrentGlobalSpeed == 0.0 )
                {
                    // Paused so resume playback.
                    mediaPlayer.Clock.Controller.Resume();
                    Playing = true;
                }
                else
                {
                    // Playing so pause playback.
                    mediaPlayer.Clock.Controller.Pause();
                    Playing = false;
                }
            }
            else if ( mediaPlayer.Clock.CurrentState == System.Windows.Media.Animation.ClockState.Stopped )
            {
                // Stopped so begin playback.
                mediaPlayer.Clock.Controller.Begin();
                Playing = true;
            }
        }

        private void NextClick( object sender, RoutedEventArgs e )
        {
            Pasted = false;
            Playing = false;
            IFeatureExtension ourFeature = stepDetails.TheFeatureManager.ActiveFeature;

            var cb = ourFeature.Commands.FindByName("AdornmentGatewayCB");

            BlackboardManager.Current.Set("AdornmentCommand", "CloseAndNavigate");

            cb.Evaluate();
            cb.Value.Execute();
            //this.PasteButton.Visibility = System.Windows.Visibility.Visible;
        }

        private void PnPClick( object sender, RoutedEventArgs e )
        {
            // TODO: Show information about patterns and practices.
        }

        private void SettingsClick( object sender, RoutedEventArgs e )
        {
            // TODO: Show the In-The-Box settings manager.
        }

        public event PropertyChangedEventHandler  PropertyChanged;

        private void NotifyPropertyChanged( string propertyName )
        {
            if ( PropertyChanged != null )
            {
                PropertyChanged( this, new PropertyChangedEventArgs( propertyName ) );
            }
        }
    }
}
